@@ -188,6 +188,8 @@ def do_plan_delete(cs, args):
            help="Id of plan to update.")
 @utils.arg("--name", metavar="<name>",
            help="A name to which the plan will be renamed.")
+@utils.arg("--description", metavar="<description>",
+           help="Description to which the plan will be updated.")
 @utils.arg("--resources", metavar="<id=type=name,id=type=name>",
            help="Resources to which the plan will be updated.")
 @utils.arg("--status", metavar="<suspended|started>",
@@ -197,6 +199,8 @@ def do_plan_update(cs, args):
     data = {}
     if args.name is not None:
         data['name'] = args.name
+    if args.description is not None:
+            data['description'] = args.description
     if args.resources is not None:
         plan_resources = arg_utils.extract_resources(args)
         data['resources'] = plan_resources
@@ -365,6 +369,140 @@ def do_restore_show(cs, args):
     utils.print_dict(restore.to_dict(), dict_format_list=dict_format_list)
 
 
+@utils.arg('provider_id',
+           metavar='<provider_id>',
+           help='Provider id.')
+@utils.arg('checkpoint_id',
+           metavar='<checkpoint_id>',
+           help='Checkpoint id.')
+@utils.arg('--parameters-json',
+           type=str,
+           dest='parameters_json',
+           metavar='<parameters>',
+           default=None,
+           help='Verification parameters in json format.')
+@utils.arg('--parameters',
+           action='append',
+           metavar='resource_type=<type>[,resource_id=<id>,key=val,...]',
+           default=[],
+           help='Verification parameters, may be specified multiple times. '
+           'resource_type: type of resource to apply parameters. '
+           'resource_id: limit the parameters to a specific resource. '
+           'Other keys and values: according to provider\'s schema.'
+           )
+def do_verification_create(cs, args):
+    """Creates a verification."""
+    if not uuidutils.is_uuid_like(args.provider_id):
+        raise exceptions.CommandError(
+            "Invalid provider id provided.")
+
+    if not uuidutils.is_uuid_like(args.checkpoint_id):
+        raise exceptions.CommandError(
+            "Invalid checkpoint id provided.")
+
+    verification_parameters = arg_utils.extract_parameters(args)
+    verification = cs.verifications.create(args.provider_id,
+                                           args.checkpoint_id,
+                                           verification_parameters)
+    dict_format_list = {"parameters"}
+    utils.print_dict(verification.to_dict(), dict_format_list=dict_format_list)
+
+
+@utils.arg('--all-tenants',
+           dest='all_tenants',
+           metavar='<0|1>',
+           nargs='?',
+           type=int,
+           const=1,
+           default=0,
+           help='Shows details for all tenants. Admin only.')
+@utils.arg('--all_tenants',
+           nargs='?',
+           type=int,
+           const=1,
+           help=argparse.SUPPRESS)
+@utils.arg('--status',
+           metavar='<status>',
+           default=None,
+           help='Filters results by a status. Default=None.')
+@utils.arg('--marker',
+           metavar='<marker>',
+           default=None,
+           help='Begin returning verifications that appear later in the'
+                'list than that represented by this verification id. '
+                'Default=None.')
+@utils.arg('--limit',
+           metavar='<limit>',
+           default=None,
+           help='Maximum number of verifications to return. Default=None.')
+@utils.arg('--sort_key',
+           metavar='<sort_key>',
+           default=None,
+           help=argparse.SUPPRESS)
+@utils.arg('--sort_dir',
+           metavar='<sort_dir>',
+           default=None,
+           help=argparse.SUPPRESS)
+@utils.arg('--sort',
+           metavar='<key>[:<direction>]',
+           default=None,
+           help=(('Comma-separated list of sort keys and directions in the '
+                  'form of <key>[:<asc|desc>]. '
+                  'Valid keys: %s. '
+                  'Default=None.') % ', '.join(base.SORT_KEY_VALUES)))
+@utils.arg('--tenant',
+           type=str,
+           dest='tenant',
+           nargs='?',
+           metavar='<tenant>',
+           help='Display information from single tenant (Admin only).')
+def do_verification_list(cs, args):
+    """Lists all verifications."""
+
+    all_tenants = 1 if args.tenant else \
+        int(os.environ.get("ALL_TENANTS", args.all_tenants))
+    search_opts = {
+        'all_tenants': all_tenants,
+        'project_id': args.tenant,
+        'status': args.status,
+    }
+
+    if args.sort and (args.sort_key or args.sort_dir):
+        raise exceptions.CommandError(
+            'The --sort_key and --sort_dir arguments are '
+            'not supported with --sort.')
+
+    verifications = cs.verifications.list(search_opts=search_opts,
+                                          marker=args.marker,
+                                          limit=args.limit,
+                                          sort_key=args.sort_key,
+                                          sort_dir=args.sort_dir,
+                                          sort=args.sort)
+
+    key_list = ['Id', 'Project id', 'Provider id', 'Checkpoint id',
+                'Parameters', 'Status']
+
+    if args.sort_key or args.sort_dir or args.sort:
+        sortby_index = None
+    else:
+        sortby_index = 0
+    formatters = {"Parameters": lambda obj: json.dumps(
+        obj.parameters, indent=2, sort_keys=True)}
+    utils.print_list(verifications, key_list, exclude_unavailable=True,
+                     sortby_index=sortby_index, formatters=formatters)
+
+
+@utils.arg('verification',
+           metavar='<verification>',
+           help='ID of verification.')
+def do_verification_show(cs, args):
+    """Shows verification details."""
+    verification = cs.verifications.get(args.verification)
+    dict_format_list = {"parameters"}
+    utils.print_dict(verification.to_dict(),
+                     dict_format_list=dict_format_list)
+
+
 def do_protectable_list(cs, args):
     """Lists all protectable types."""
 
@@ -578,6 +716,19 @@ def do_checkpoint_create(cs, args):
                      json_format_list=json_format_list)
 
 
+@utils.arg('--all-tenants',
+           dest='all_tenants',
+           metavar='<0|1>',
+           nargs='?',
+           type=int,
+           const=1,
+           default=0,
+           help='Shows details for all tenants. Admin only.')
+@utils.arg('--all_tenants',
+           nargs='?',
+           type=int,
+           const=1,
+           help=argparse.SUPPRESS)
 @utils.arg('provider_id',
            metavar='<provider_id>',
            help='ID of provider.')
@@ -651,6 +802,7 @@ def do_checkpoint_list(cs, args):
         'start_date': args.start_date,
         'end_date': args.end_date,
         'project_id': args.project_id,
+        'all_tenants': args.all_tenants
     }
 
     if args.sort and (args.sort_key or args.sort_dir):
@@ -670,8 +822,11 @@ def do_checkpoint_list(cs, args):
         sortby_index = None
     else:
         sortby_index = 0
-    formatters = {"Protection plan": lambda obj: json.dumps(
-        obj.protection_plan, indent=2, sort_keys=True)}
+
+    def plan_formatter(obj):
+        return "Name: %s\nId: %s" % (obj.protection_plan['name'],
+                                     obj.protection_plan['id'])
+    formatters = {"Protection plan": plan_formatter}
     utils.print_list(checkpoints, key_list, exclude_unavailable=True,
                      sortby_index=sortby_index, formatters=formatters)
 
@@ -715,6 +870,45 @@ def do_checkpoint_delete(cs, args):
                                       "specified checkpoint.")
 
 
+@utils.arg('provider_id',
+           metavar='<provider_id>',
+           help='Id of provider.')
+@utils.arg('checkpoint',
+           metavar='<checkpoint>',
+           nargs="+",
+           help='ID of checkpoint.')
+@utils.arg('--available',
+           action='store_const',
+           dest='state',
+           default='error',
+           const='available',
+           help='Request the checkpoint be reset to "available" state instead '
+                'of "error" state(the default).')
+def do_checkpoint_reset_state(cs, args):
+    """Reset state of a checkpoint."""
+    failure_count = 0
+
+    for checkpoint_id in args.checkpoint:
+        try:
+            cs.checkpoints.reset_state(args.provider_id, checkpoint_id,
+                                       args.state)
+        except exceptions.NotFound:
+            failure_count += 1
+            print("Failed to reset state of '{0}'; checkpoint not found".
+                  format(checkpoint_id))
+        except exceptions.Forbidden:
+            failure_count += 1
+            print("Failed to reset state of '{0}'; not allowed".
+                  format(checkpoint_id))
+        except exceptions.BadRequest:
+            failure_count += 1
+            print("Failed to reset state of '{0}'; invalid input or "
+                  "current checkpoint state".format(checkpoint_id))
+    if failure_count == len(args.checkpoint):
+        raise exceptions.CommandError("Unable to find or reset any of the "
+                                      "specified checkpoint's state.")
+
+
 @utils.arg('--all-tenants',
            dest='all_tenants',
            metavar='<0|1>',
@@ -833,7 +1027,8 @@ def do_trigger_update(cs, args):
     """Update a trigger."""
     trigger_info = {}
     trigger_properties = arg_utils.extract_properties(args)
-    trigger_info['name'] = args.name
+    if args.name:
+        trigger_info['name'] = args.name
     trigger_info['properties'] = trigger_properties
     trigger = cs.triggers.update(args.trigger_id, trigger_info)
     dict_format_list = {"properties"}
@@ -969,7 +1164,9 @@ def do_scheduledoperation_list(cs, args):
            help='Trigger name.')
 @utils.arg('operation_type',
            metavar='<operation_type>',
-           help='Operation Type of scheduled operation.')
+           choices=['protect', 'retention_protect'],
+           help='Operation Type of scheduled operation. Valid values are '
+                '"protect" or "retention_protect."')
 @utils.arg('trigger_id',
            metavar='<trigger_id>',
            help='Trigger id of scheduled operation.')
@@ -1108,3 +1305,142 @@ def do_operationlog_show(cs, args):
     """Shows operation_log details."""
     operation_log = cs.operation_logs.get(args.operation_log)
     utils.print_dict(operation_log.to_dict())
+
+
+@utils.arg('--host',
+           metavar='<hostname>',
+           default=None,
+           help='Name of host.')
+@utils.arg('--binary',
+           metavar='<binary>',
+           default=None,
+           help='Service binary.')
+def do_service_list(cs, args):
+    """Show a list of all running services. Filter by host & binary."""
+    result = cs.services.list(host=args.host, binary=args.binary)
+    columns = ["Id", "Binary", "Host", "Status", "State",
+               "Updated_at", "Disabled Reason"]
+    utils.print_list(result, columns)
+
+
+@utils.arg('service_id',
+           metavar='<service_id>',
+           help='ID of the service.')
+def do_service_enable(cs, args):
+    """Enable the service."""
+    result = cs.services.enable(args.service_id)
+    utils.print_list([result], ["Id", "Binary", "Host", "Status", "State",
+                                "Updated_at", "Disabled Reason"])
+
+
+@utils.arg('service_id',
+           metavar='<service_id>',
+           help='ID of the service.')
+@utils.arg('--reason',
+           metavar='<reason>',
+           help='Reason for disabling the service.')
+def do_service_disable(cs, args):
+    """Disable the service"""
+    if args.reason:
+        result = cs.services.disable_log_reason(args.service_id, args.reason)
+    else:
+        result = cs.services.disable(args.service_id)
+    utils.print_list([result], ["Id", "Binary", "Host", "Status", "State",
+                                "Updated_at", "Disabled Reason"])
+
+
+@utils.arg(
+    '--tenant',
+    metavar='<tenant>',
+    default=None,
+    help='ID of tenant to list the quotas for.')
+@utils.arg(
+    '--detail',
+    action='store_true',
+    help='Optional flag to indicate whether to show quota in detail. '
+         'Default false.')
+def do_quota_show(cs, args):
+    """List the quotas for a tenant."""
+    project_id = args.tenant or cs.http_client.get_project_id()
+    kwargs = {
+        "project_id": project_id,
+        "detail": args.detail,
+    }
+    result = cs.quotas.get(**kwargs)
+    _quota_set_pretty_show(result)
+
+
+def _quota_set_pretty_show(quotas):
+    """Convert quotas object to dict and display."""
+
+    new_quotas = {}
+    for quota_k, quota_v in sorted(quotas.to_dict().items()):
+        if isinstance(quota_v, dict):
+            quota_v = '\n'.join(
+                ['%s = %s' % (k, v) for k, v in sorted(quota_v.items())])
+        new_quotas[quota_k] = quota_v
+
+    utils.print_dict(new_quotas)
+
+
+@utils.arg(
+    'tenant',
+    metavar='<tenant>',
+    help='ID of tenant to set the quotas for.')
+@utils.arg(
+    '--plans',
+    metavar='<plans>',
+    type=int,
+    default=None,
+    help='New value for the "plans" quota.')
+def do_quota_update(cs, args):
+    """Update the quotas for a project (Admin only)."""
+    project_id = args.tenant
+    data = {
+        "plans": args.plans,
+    }
+    result = cs.quotas.update(project_id, data)
+    _quota_set_pretty_show(result)
+
+
+@utils.arg(
+    '--tenant',
+    metavar='<tenant>',
+    default=None,
+    help='ID of tenant to list the quotas for.')
+def do_quota_defaults(cs, args):
+    """List the default quotas for a tenant."""
+    project_id = args.tenant or cs.http_client.get_project_id()
+
+    result = cs.quotas.defaults(project_id)
+    _quota_set_pretty_show(result)
+
+
+@utils.arg(
+    'class_name',
+    metavar='<class_name>',
+    help='Name of quota class to list the quotas for.')
+def do_quota_class_show(cs, args):
+    """List the quotas for a quota class."""
+    result = cs.quota_classes.get(args.class_name)
+    _quota_set_pretty_show(result)
+
+
+@utils.arg(
+    'class_name',
+    metavar='<class_name>',
+    help='Name of quota class to set the quotas for.')
+@utils.arg(
+    '--plans',
+    metavar='<plans>',
+    type=int,
+    default=None,
+    help='New value for the "plans" quota.')
+def do_quota_class_update(cs, args):
+    """Update the quotas for a quota class (Admin only)."""
+    class_name = args.class_name
+    data = {
+        "plans": args.plans,
+    }
+    result = cs.quota_classes.update(class_name, data)
+    _quota_set_pretty_show(result)
